import { 
  select, 
  csv,
	scaleOrdinal,
  pie,
	arc,
  getBBox,
	sum} from 'd3';

function pointIsInArc(pt, ptData, d3Arc) {
  // Center of the arc is assumed to be 0,0
  // (pt.x, pt.y) are assumed to be relative to the center
  var r1 = d3Arc.innerRadius()(ptData), // Note: Using the innerRadius
      r2 = d3Arc.outerRadius()(ptData),
      theta1 = d3Arc.startAngle()(ptData),
      theta2 = d3Arc.endAngle()(ptData);

  var dist = pt.x * pt.x + pt.y * pt.y,
      angle = Math.atan2(pt.x, -pt.y); // Note: different coordinate system.

  angle = (angle < 0) ? (angle + Math.PI * 2) : angle;

  return (r1 * r1 <= dist) && (dist <= r2 * r2) && 
         (theta1 <= angle) && (angle <= theta2);
}

export const renderAirportDepartureDelay = (data, selector) => {
  const width = 190;
	const height = 210;
	// const width = 960;
	// const height = 400;
	const radius = Math.min(width, height) / 2;
  const title = 'Delay per Airport';
  
  const plot_container = select('.' + selector)
  	.append("div")
  	.attr("class", "helper_div")

 	const plotVariable = d => d.DEPARTURE_DELAY;
  
  const colorScale = scaleOrdinal().domain(['1', '2', '3', '4', '5', '6', '7', '8']).range(['#0277bd','#0fa4fc','#039cf7','#0397ef','#0287d6','#0277bd','#026baa','#025f96']);
  // const colorScale = scaleOrdinal().domain(['1', '2', '3', '4', '5', '6', '7', '8', '9']).range(['#039cf7','#028fe4','#0283d0','#0277bd','#026baa','#025f96','#015283','#01466f','#013a5c']);
  
  // const colorScale = scaleOrdinal().domain(['1', '2', '3', '4', '5', '6', '7', '8', '9']).range(['#039cf7','#028fe4','#0283d0','#0277bd','#026baa','#025f96','#015283','#01466f','#013a5c']);
  // '#0277BD','#025F97','#0a7fc5','#1287cd','#188dd3','#2196dc','#2ba0e6','#33a8ee',
	//   range(['#039cf7','#028fe4','#0283d0','#0277bd','#026baa','#025f96','#015283','#01466f','#013a5c']);  
  
	// const colorScale = scaleOrdinal()
  // 	.domain(['>15mins', '<15mins'])
  // 	.range(['#ff6309','#0277bd']);
  // 	// .range(['#3e3e97','#838690','#e56d26']); 
    // #ff6361
  
  const pie_calc = pie()
  	.value(plotVariable)(data);  
  
	const arc = d3.arc()
		.outerRadius(radius - 10)
		.innerRadius(radius - 85) //radius - 105
  	.padAngle([1/60]) // as angle is specified in radians
  	.cornerRadius([10]);

	const labelArc = d3.arc()
		.outerRadius(radius - 60)
		.innerRadius(radius - 40);
    
  const g = plot_container
  .append('svg')
  	.attr('width', width)
  	.attr('height', height)
  .selectAll('arc').append('g')
  	.data(pie_calc)
  	.enter().append('g')
  	.attr('transform', `translate(${width/2}, ${height/2})`)
  	.attr('class', 'pie-slice');
  
  g.append('path')
		.attr('d', arc)
		.style('fill', d => colorScale(d.data.ORIGIN_AIRPORT));
  
  g.append('text')
  	.attr('transform', d => 
         `translate(${labelArc.centroid(d)})`)
    .text(d => d.data.ORIGIN_AIRPORT)
    .each(function (d) {
       var bb = this.getBBox(),
           center = arc.centroid(d);

       var topLeft = {
         x : center[0] + bb.x,
         y : center[1] + bb.y
       };

       var topRight = {
         x : topLeft.x + bb.width,
         y : topLeft.y
       };

       var bottomLeft = {
         x : topLeft.x,
         y : topLeft.y + bb.height
       };

       var bottomRight = {
         x : topLeft.x + bb.width,
         y : topLeft.y + bb.height
       };
    	 
       d.visible = pointIsInArc(topLeft, d, arc) &&
                   pointIsInArc(topRight, d, arc) &&
                   pointIsInArc(bottomLeft, d, arc) &&
                   pointIsInArc(bottomRight, d, arc);			
    })
    .style('display', function (d) { return d.visible ? null : "none"; })
  	.attr('class', 'dounat-text');
    
  g.append('text')
    .attr('class', 'title')
    .attr('y', 100)
  	.attr('x', -55)
    .text(title);
  
  g.on("mouseover", d => {
    
    let total = sum(data.map(plotVariable))
    
    let percent = Math.round(1000 * d.data.DEPARTURE_DELAY / total) / 10;
    
    d3.select('#tooltip')
        .style('left', d3.event.pageX + 'px')
        .style('top', d3.event.pageY + 'px')
        .style('opacity', 1)
        .select('#value')
        .text(percent);
	})
    g.on('mouseout', () => {
      // Hide the tooltip
      d3.select('#tooltip')
        .style('opacity', 0);
		});
  
  // prevent tooltip to disappear when hovered 
  select('#tooltip').
  	on('mouseover', () => {
  		select('#tooltip').style('opacity', 1)
  });
  
}

csv('https://gist.githubusercontent.com/josvin92/9a6b89cb9365b1042d6648fc584c3198/raw/Flight_Delay.csv').then(data => {
  data.forEach(d => {
    d.DEPARTURE_DELAY = +d.DEPARTURE_DELAY; 
			// console.log(d.DEPARTURE_DELAY);
  });
  
  const dataByAirportDelay = d3.nest()
  	.key(function(d) {return d.ORIGIN_AIRPORT;})
  	.rollup(function(v){return d3.sum(v, function(d) { return d.DEPARTURE_DELAY; });})
  	.entries(data)
  	.map(d => {
    	return {
        ORIGIN_AIRPORT: d.key,
        DEPARTURE_DELAY: d.value}
      })
    
  // dataByAirlineDelay.forEach(function(element) {
  //     console.log(element);
  //   });
// console.log(JSON.stringify(dataByDelay15));  
  render(dataByAirportDelay, 'plot_container');
});