import {
  select,
  csv,
  scaleLinear,
  max,
  min,
  scaleBand,
  axisLeft,
  axisBottom,
  format,
  tip,
} from 'd3';

const width = 500;
const height = 250;
export const renderAverageDelayReason = (data, reason, maxTotal, selector) => {
  const title = 'Average Delay Time and Delay Reason for various carriers';

  const xValue = (d) => d.AIRLINE;
  const xAxisLabel = 'AIRLINE';

  const yValue = (d) => d.DEPARTURE_DELAY;
  const yAxisLabel = 'DEPARTURE_DELAY';
  
  const colorValue = (d) => d.CANCELLATION_REASON;

	var groups = d3.map(data, function(d){return(d.AIRLINE)}).keys()  
	
  // var subgroups = ['Carrier Delay','Late Aircraft Delay','National Air System Delay','Security Delay','Weather Delay']
  
  // console.log(data);
  // console.log(subgroups);
  
  const margin = { top: 35, right: 60, bottom: 0, left: 50 };
  // const margin = { top: 50, right: 40, bottom: 75, left: 105 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const xScale = scaleBand()
    .domain(groups)
    .range([0, innerWidth])
    .padding(0.1);

  const yScale = scaleLinear()
    .domain([0, maxTotal])
  // .domain([0, min(maxTotal, yValue)])
    .range([innerHeight, 0])
    .nice();

	// color palette = one color per subgroup
  var color = d3.scaleOrdinal()
  .domain(reason)
  .range(d3.schemeCategory10);
   
  // append the svg object to the body of the page
	const svg = d3.select('.' + selector)
  	.append("svg")
    .attr("width", width + margin.left + margin.right)
    .attr("height", height + margin.top + margin.bottom);

  const g = svg
    .append('g')
    .attr('transform', `translate(${margin.left}, ${margin.top})`);

  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(4);

  const xAxis = axisBottom(xScale)
  	.tickSize(-innerHeight)
  	.tickPadding(5);

  const yAxisG = g.append('g').call(yAxis);

  // yAxisG.selectAll('.tick line').remove();

  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -30)
    .attr('x', -innerHeight / 2)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);
  
  const xAxisG = g
    .append('g')
    .call(xAxis)   
  	.attr('transform', `translate(0, ${innerHeight})`);

  // xAxisG.select('.domain').remove();

  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 25)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .attr('text-anchor', 'middle')
    .text(xAxisLabel);
            
  g.append("g")
  .selectAll("g")    	
  .data(data)   
  .enter()
  .append("g")  
  .selectAll("rect")
  // enter a second time = loop subgroup per subgroup to add all rectangles
  .data(function(d) { return d.bars; })
    .enter()
    .append("rect")
    .attr("x", function(d) { return xScale(xValue(d)); })
    .attr("y", function(d) { return yScale(d.y1); })
    .attr("height", function(d) { return Math.abs( yScale(d.y1) - yScale(d.y0) ); })
    .attr("width", xScale.bandwidth())
    .style("fill", function(d) { return color(d.CANCELLATION_REASON); })
    
  .on("mouseover", (d) => { 
        d3.select('#tooltip_delay_reason')
            .style('left', d3.event.pageX + 'px')
            .style('top', d3.event.pageY + 'px')
            .style('opacity', 1)
            .select('#value')            
      			.html("<div style='font-size:12px'> <b>Delay Reason:</b> " + colorValue(d) + "<br><b>Average Delay:</b> " + d3.format(",.2f")(yValue(d))+"</div>");
      })
  .on('mouseout', () => {
      // Hide the tooltip
      d3.select('#tooltip_delay_reason')
        .style('opacity', 0);
  });
  
  // prevent tooltip to disappear when hovered 
  select('#tooltip_delay_reason').
  	on('mouseover', () => {
  		select('#tooltip_delay_reason').style('opacity', 1)
  });
  
  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
  	.attr('x', -15)
    .text(title);
  
};

csv(
  'https://gist.githubusercontent.com/josvin92/9a6b89cb9365b1042d6648fc584c3198/raw/Flight_Delay.csv'
).then((data) => {
  data.forEach((d) => {
    d.DEPARTURE_DELAY = +d.DEPARTURE_DELAY;
  });

  const dataBySumDelay = d3
    .nest()
    .key(function (d) {
      return d.AIRLINE;
    })
    .key(function (d) {
      return d.CANCELLATION_REASON;
    })
    .rollup(function(v){return d3.mean(v, function(d) { return d.DEPARTURE_DELAY; });})
    .entries(data)
    .map((d) => {      
      return {        
        AIRLINE: d.key,
        values: (d.values).map( function (d) {
           return {
                CANCELLATION_REASON: d.key,
                DEPARTURE_DELAY: d.value,                
            };        
        }).sort(function (a, b) {
      return d3.descending(a.CANCELLATION_REASON, b.CANCELLATION_REASON);
    })
      
      };
    })
  .sort(function (a, b) {
      return d3.ascending(a.AIRLINE, b.AIRLINE);
    });

var maxTotal = 0; //maximum count per airline, 
                  //for setting the y domain
dataBySumDelay.forEach(function(airlineGroup) {
   //for each entry in the nested array,
   //each of which contains all the rows for a given airline,
   //calculate the total count,
   //and the before-and-after counts for each category.  

    airlineGroup.bars = [];
        //create an array to hold one value for each bar
        //(i.e., two values for each of the original rows)

    var total = 0; //total count per airline
		
  	// airlineGroup.values.sort(function (a, b) {
  	// return d3.ascending(+a.CANCELLATION_REASON, +b.CANCELLATION_REASON);
  	// })

    airlineGroup.values.forEach(function(row) {
        //the values array created by the nest function
        //contians all the original row data objects
        //that match this airline (i.e., the nesting key)

        //create an object representing the bar for
        //the scanned count, and add to the bars array
        airlineGroup.bars.push(
            {AIRLINE: airlineGroup.AIRLINE,
             CANCELLATION_REASON: row.CANCELLATION_REASON,
             DEPARTURE_DELAY: +row.DEPARTURE_DELAY,
             y0: total, //total value *before* this bar
             y1: (total = total + +row.DEPARTURE_DELAY) //new total
            }
            );
    });

    maxTotal = Math.max(maxTotal, total); //update max

});

// console.log(dataBySumDelay);
// Used for flattening the nested array  
   var flat_CANCELLATION_REASON = []
   dataBySumDelay.forEach(function(airline) {     
      airline.values.forEach(function(airlineDelay) {        
        flat_CANCELLATION_REASON.push(
          airlineDelay.CANCELLATION_REASON,          
        );
      });
    });
  flat_CANCELLATION_REASON = d3.set(flat_CANCELLATION_REASON).values().sort(function (a, b) {
      return d3.ascending(a, b);
    });
  	console.log(flat_CANCELLATION_REASON);

// Used for flattening the nested array  
//    var flatDataBySumDelay = []
//    dataBySumDelay.forEach(function(airline) {     
//       airline.values.forEach(function(airlineDelay) {        
//         flatDataBySumDelay.push({
//           AIRLINE: airline.AIRLINE,
//           CANCELLATION_REASON: airlineDelay.CANCELLATION_REASON,
//           DEPARTURE_DELAY: airlineDelay.DEPARTURE_DELAY
//         });
//       });
//     });
//   	console.log(flatDataBySumDelay);
//   	// console.log(dataBySumDelay[0]);

  // console.log(JSON.stringify(dataByCountDestination));
  render(dataBySumDelay, flat_CANCELLATION_REASON, maxTotal);
});
