import { 
  select, 
  csv,
	scaleOrdinal,
	pie,
	arc,
	sum} from 'd3';

export const renderDepartureDelay = (data, selector) => {
  const width = 190;
	const height = 210;
	// const width = 960;
	// const height = 400;
	const radius = Math.min(width, height) / 2;
  
  const title = 'Departure Delay';
  
  const plot_container = select('.' + selector)
  	.append("div")
  	.attr("class", "helper_div")

 	const plotVariable = d => d.DEPARTURE_DELAY;
  
  const colorScale = scaleOrdinal()
  	.domain(['>15mins', '<15mins'])
  	.range(['#ff6309','#0277bd']);
  	// .range(['#3e3e97','#838690','#e56d26']); 
    // #ff6361
  
  const pie_calc = pie()
  	.value(plotVariable)(data);  
  
const arc = d3.arc()
		.outerRadius(radius - 10)
		.innerRadius(radius - 85) //radius - 105
  	.padAngle([1/60]) // as angle is specified in radians
  	.cornerRadius([10]);

	const labelArc = d3.arc()
		.outerRadius(radius - 60)
		.innerRadius(radius - 40);
    
  const g = plot_container
  .append('svg')
  	.attr('width', width)
  	.attr('height', height)
  .selectAll('arc').append('g')
  	.data(pie_calc)
  	.enter().append('g')
  	.attr('transform', `translate(${width/2}, ${height/2})`)
  	.attr('class', 'pie-slice');
  
  g.append('path')
		.attr('d', arc)
		.style('fill', d => colorScale(d.data.DEPARTURE_DELAY_15));
  
  g.append('text')
  	.attr('transform', d => 
         `translate(${labelArc.centroid(d)})`)
    .text(d => d.data.DEPARTURE_DELAY_15)
  	.attr('class', 'dounat-text');
  
  g.append('text')
    .attr('class', 'title')
    .attr('y', 100)
  	.attr('x', -55)
    .text(title);
    
  g.on("mouseover", d => {
    
    let total = sum(data.map(plotVariable))
    
    let percent = Math.round(1000 * d.data.DEPARTURE_DELAY / total) / 10;
    
    d3.select('#tooltip')
        .style('left', d3.event.pageX + 'px')
        .style('top', d3.event.pageY + 'px')
        .style('opacity', 1)
        .select('#value')
        .text(percent);
	})
    g.on('mouseout', () => {
      // Hide the tooltip
      d3.select('#tooltip')
        .style('opacity', 0);
		});
  
  // prevent tooltip to disappear when hovered 
  select('#tooltip').
  	on('mouseover', () => {
  		select('#tooltip').style('opacity', 1)
  });
  
}
