import {
  select,
  csv,
  scaleLinear,
  max,
  scaleBand,
  axisLeft,
  axisBottom,
  format,
  tip,
} from 'd3';

// const svg = select('svg');

// const width = +svg.attr('width');
// const height = +svg.attr('height');

const width = 450;
const height = 250;

export const rendertopDestination = (data, selector) => {
  const title = 'Top 15 Destination Airports';

  const xValue = (d) => d.Count;
  const xAxisLabel = 'Number of flights to Destination Airport';

  const yValue = (d) => d.Destination_Airport;
  const yAxisLabel = 'Destination Airport';

  const margin = { top: 35, right: 40, bottom: 0, left: 50 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const xScale = scaleLinear()
    .domain([0, max(data, xValue)])
    .range([0, innerWidth])
    .nice();

  const yScale = scaleBand()
    .domain(data.map(yValue))
    .range([0, innerHeight])
    .padding(0.2);

	const svg = d3.select('.' + selector)
  	.append("svg")
    .attr("width", width + margin.left + margin.right)
    .attr("height", height + margin.top + margin.bottom);

  const g = svg
    .append('g')
    .attr('transform', `translate(${margin.left}, ${margin.top})`);

  const xAxisTickFormat = (number) => format('.1s')(number);

  const xAxis = axisBottom(xScale)
    .tickFormat(xAxisTickFormat)
    .tickSize(-innerHeight)
    .tickPadding(5);

  const yAxis = axisLeft(yScale).tickSize(-innerWidth).tickPadding(4);

  const yAxisG = g.append('g').call(yAxis);

  yAxisG.selectAll('.tick line').remove();

  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -40)
    .attr('x', -innerHeight / 2)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);
  
  const xAxisG = g
    .append('g')
    .call(xAxis)   
  	.attr('transform', `translate(0, ${innerHeight})`);

  // xAxisG.select('.domain').remove();

  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 25)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .attr('text-anchor', 'middle')
    .text(xAxisLabel);
  
  g.selectAll('rect')
    .data(data)
    .enter()
    .append('rect')
    .attr('y', (d) => yScale(yValue(d)))
    .attr('width', (d) => xScale(xValue(d)))
    .attr('height', yScale.bandwidth())
    .on("mouseover", (d) => {    
          d3.select('#tooltip_bar')
              .style('left', d3.event.pageX + 'px')
              .style('top', d3.event.pageY + 'px')
              .style('opacity', 1)
              .select('#value2')
              .text(xValue(d));
        })
    .on('mouseout', () => {
        // Hide the tooltip
        d3.select('#tooltip_bar')
          .style('opacity', 0);
		});
  
  // prevent tooltip to disappear when hovered 
  select('#tooltip_bar').
  	on('mouseover', () => {
  		select('#tooltip_bar').style('opacity', 1)
  });
  
  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
  	.attr('x', 50)
    .text(title);
  
};

