import { csv } from 'd3';

import { renderArrivalDelay } from './ArrivalDelay/index.js';
import { renderDepartureDelay } from './DepartureDelay/index.js';
import { renderAirlineDepartureDelay } from './AirlineDepartureDelay/index.js';
import { renderAirportDepartureDelay } from './AirportDepartureDelay/index.js';
import { rendertopDestination } from './TopDestinationAirports/index.js';
import { renderAverageDelayReason } from './AverageDelayTimeReason/index.js';

csv('https://gist.githubusercontent.com/josvin92/9a6b89cb9365b1042d6648fc584c3198/raw/Flight_Delay.csv').then(data => {
  data.forEach(d => {
    if(d.ARRIVAL_DELAY_15 == 1){
    		d.ARRIVAL_DELAY_15 = ">15 mins"; 
    }
    else if(d.ARRIVAL_DELAY_15 == 0){
    		d.ARRIVAL_DELAY_15 = "<15 mins"; 
    }  	
    
    if(d.DEPARTURE_DELAY_15 == 1){
    		d.DEPARTURE_DELAY_15 = ">15 mins"; 
    }
    else if(d.DEPARTURE_DELAY_15 == 0){
    		d.DEPARTURE_DELAY_15 = "<15 mins"; 
    }
  	d.ARRIVAL_DELAY = +d.ARRIVAL_DELAY; 
    d.DEPARTURE_DELAY = +d.DEPARTURE_DELAY; 
			
  });

  const dataByArrivalDelay15 = d3.nest()
  	.key(function(d) {return d.ARRIVAL_DELAY_15;})
  	.rollup(function(v){return d3.sum(v, function(d) { return d.ARRIVAL_DELAY; });})
  	.entries(data)
  	.map(d => {
    	return {
        ARRIVAL_DELAY_15: d.key,
        ARRIVAL_DELAY: d.value}
      })

  
  const dataByDepartureDelay15 = d3.nest()
  	.key(function(d) {return d.DEPARTURE_DELAY_15;})
  	.rollup(function(v){return d3.sum(v, function(d) { return d.DEPARTURE_DELAY; });})
  	.entries(data)
  	.map(d => {
    	return {
        DEPARTURE_DELAY_15: d.key,
        DEPARTURE_DELAY: d.value}
      })
  
  const dataByAirlineDelay = d3.nest()
  	.key(function(d) {return d.AIRLINE;})
  	.rollup(function(v){return d3.sum(v, function(d) { return d.DEPARTURE_DELAY; });})
  	.entries(data)
  	.map(d => {
    	return {
        AIRLINE: d.key,
        DEPARTURE_DELAY: d.value}
      })
  
  const dataByAirportDelay = d3.nest()
  	.key(function(d) {return d.ORIGIN_AIRPORT;})
  	.rollup(function(v){return d3.sum(v, function(d) { return d.DEPARTURE_DELAY; });})
  	.entries(data)
  	.map(d => {
    	return {
        ORIGIN_AIRPORT: d.key,
        DEPARTURE_DELAY: d.value}
      })
  
  const dataByCountDestination = d3
    .nest()
    .key(function (d) {
      return d.DESTINATION_AIRPORT;
    })
    .rollup(function (v) {
      return v.length;
    })
    .entries(data)
    .map((d) => {
      return {
        Destination_Airport: d.key,
        Count: d.value,
      };
    });
  var topDataByCountDestination = dataByCountDestination
    .sort(function (a, b) {
      return d3.descending(+a.Count, +b.Count);
    })
    .slice(0, 15); //top 15 here

const dataBySumDelay = d3
    .nest()
    .key(function (d) {
      return d.AIRLINE;
    })
    .key(function (d) {
      return d.CANCELLATION_REASON;
    })
    .rollup(function(v){return d3.mean(v, function(d) { return d.DEPARTURE_DELAY; });})
    .entries(data)
    .map((d) => {      
      return {        
        AIRLINE: d.key,
        values: (d.values).map( function (d) {
           return {
                CANCELLATION_REASON: d.key,
                DEPARTURE_DELAY: d.value,                
            };        
        }).sort(function (a, b) {
      return d3.descending(a.CANCELLATION_REASON, b.CANCELLATION_REASON);
    })
      
      };
    })
  .sort(function (a, b) {
      return d3.ascending(a.AIRLINE, b.AIRLINE);
    });

var maxTotal = 0; //maximum count per airline, 
                  //for setting the y domain
dataBySumDelay.forEach(function(airlineGroup) {
   //for each entry in the nested array,
   //each of which contains all the rows for a given airline,
   //calculate the total count,
   //and the before-and-after counts for each category.  

    airlineGroup.bars = [];
        //create an array to hold one value for each bar
        //(i.e., two values for each of the original rows)

    var total = 0; //total count per airline
		
  	// airlineGroup.values.sort(function (a, b) {
  	// return d3.ascending(+a.CANCELLATION_REASON, +b.CANCELLATION_REASON);
  	// })

    airlineGroup.values.forEach(function(row) {
        //the values array created by the nest function
        //contians all the original row data objects
        //that match this airline (i.e., the nesting key)

        //create an object representing the bar for
        //the scanned count, and add to the bars array
        airlineGroup.bars.push(
            {AIRLINE: airlineGroup.AIRLINE,
             CANCELLATION_REASON: row.CANCELLATION_REASON,
             DEPARTURE_DELAY: +row.DEPARTURE_DELAY,
             y0: total, //total value *before* this bar
             y1: (total = total + +row.DEPARTURE_DELAY) //new total
            }
            );
    });

    maxTotal = Math.max(maxTotal, total); //update max

});

// console.log(dataBySumDelay);
// Used for flattening the nested array  
   var flat_CANCELLATION_REASON = []
   dataBySumDelay.forEach(function(airline) {     
      airline.values.forEach(function(airlineDelay) {        
        flat_CANCELLATION_REASON.push(
          airlineDelay.CANCELLATION_REASON,          
        );
      });
    });
  flat_CANCELLATION_REASON = d3.set(flat_CANCELLATION_REASON).values().sort(function (a, b) {
      return d3.ascending(a, b);
    });
  	console.log(flat_CANCELLATION_REASON);

// Used for flattening the nested array  
//    var flatDataBySumDelay = []
//    dataBySumDelay.forEach(function(airline) {     
//       airline.values.forEach(function(airlineDelay) {        
//         flatDataBySumDelay.push({
//           AIRLINE: airline.AIRLINE,
//           CANCELLATION_REASON: airlineDelay.CANCELLATION_REASON,
//           DEPARTURE_DELAY: airlineDelay.DEPARTURE_DELAY
//         });
//       });
//     });
//   	console.log(flatDataBySumDelay);
//   	// console.log(dataBySumDelay[0]);

  // console.log(JSON.stringify(dataByCountDestination));  
  
  
  // dataByDelay15.forEach(function(element) {
  //     console.log(element);
  //   });
// console.log(JSON.stringify(dataByDelay15));  
  rendertopDestination(topDataByCountDestination, 'plot_container5');
  renderAverageDelayReason(dataBySumDelay, flat_CANCELLATION_REASON, maxTotal, 'plot_container6');
  renderArrivalDelay(dataByArrivalDelay15, 'plot_container1');
  renderDepartureDelay(dataByDepartureDelay15, 'plot_container2');
  renderAirlineDepartureDelay(dataByAirlineDelay, 'plot_container3');
  renderAirportDepartureDelay(dataByAirportDelay, 'plot_container4');
	
});